@@ -16,21 +16,24 @@
  */
 package org.apache.tika.io;
 
+import static java.nio.file.StandardCopyOption.REPLACE_EXISTING;
+
 import java.io.BufferedInputStream;
 import java.io.ByteArrayInputStream;
 import java.io.Closeable;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.OutputStream;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.URL;
 import java.net.URLConnection;
 import java.nio.channels.FileChannel;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.sql.Blob;
 import java.sql.SQLException;
 
@@ -85,19 +88,16 @@ public class TikaInputStream extends TaggedInputStream {
      * when you <em>don't</em> explicitly close the returned stream. The
      * recommended access pattern is:
      * <pre>
-     * TemporaryResources tmp = new TemporaryResources();
-     * try {
+     * try (TemporaryResources tmp = new TemporaryResources()) {
      *     TikaInputStream stream = TikaInputStream.get(..., tmp);
      *     // process stream but don't close it
-     * } finally {
-     *     tmp.close();
      * }
      * </pre>
      * <p>
      * The given stream instance will <em>not</em> be closed when the
-     * {@link TemporaryResources#close()} method is called. The caller
-     * is expected to explicitly close the original stream when it's no
-     * longer used.
+     * {@link TemporaryResources#close()} method is called by the
+     * try-with-resources statement. The caller is expected to explicitly
+     * close the original stream when it's no longer used.
      *
      * @since Apache Tika 0.10
      * @param stream normal input stream
@@ -113,8 +113,7 @@ public class TikaInputStream extends TaggedInputStream {
         } else {
             // Make sure that the stream is buffered and that it
             // (properly) supports the mark feature
-            if (!(stream instanceof BufferedInputStream)
-                    && !(stream instanceof ByteArrayInputStream)) {
+            if (!(stream.markSupported())) {
                 stream = new BufferedInputStream(stream);
             }
             return new TikaInputStream(stream, tmp, -1);
@@ -131,17 +130,14 @@ public class TikaInputStream extends TaggedInputStream {
      * <em>do</em> explicitly close the returned stream. The recommended
      * access pattern is:
      * <pre>
-     * TikaInputStream stream = TikaInputStream.get(...);
-     * try {
+     * try (TikaInputStream stream = TikaInputStream.get(...)) {
      *     // process stream
-     * } finally {
-     *     stream.close();
      * }
      * </pre>
      * <p>
      * The given stream instance will be closed along with any other resources
      * associated with the returned TikaInputStream instance when the
-     * {@link #close()} method is called.
+     * {@link #close()} method is called by the try-with-resources statement.
      *
      * @param stream normal input stream
      * @return a TikaInputStream instance
@@ -198,6 +194,39 @@ public class TikaInputStream extends TaggedInputStream {
                 new TemporaryResources(), data.length);
     }
 
+    /**
+     * Creates a TikaInputStream from the file at the given path.
+     * <p>
+     * Note that you must always explicitly close the returned stream to
+     * prevent leaking open file handles.
+     *
+     * @param path input file
+     * @return a TikaInputStream instance
+     * @throws IOException if an I/O error occurs
+     */
+    public static TikaInputStream get(Path path) throws IOException {
+        return get(path, new Metadata());
+    }
+
+    /**
+     * Creates a TikaInputStream from the file at the given path. The file name
+     * and length are stored as input metadata in the given metadata instance.
+     * <p>
+     * Note that you must always explicitly close the returned stream to
+     * prevent leaking open file handles.
+     *
+     * @param path input file
+     * @param metadata metadata instance
+     * @return a TikaInputStream instance
+     * @throws IOException if an I/O error occurs
+     */
+    public static TikaInputStream get(Path path, Metadata metadata)
+            throws IOException {
+        metadata.set(Metadata.RESOURCE_NAME_KEY, path.getFileName().toString());
+        metadata.set(Metadata.CONTENT_LENGTH, Long.toString(Files.size(path)));
+        return new TikaInputStream(path);
+    }
+
     /**
      * Creates a TikaInputStream from the given file.
      * <p>
@@ -207,7 +236,10 @@ public class TikaInputStream extends TaggedInputStream {
      * @param file input file
      * @return a TikaInputStream instance
      * @throws FileNotFoundException if the file does not exist
+     * @deprecated use {@link #get(Path)}. In Tika 2.0, this will be removed
+     * or modified to throw an IOException.
      */
+    @Deprecated
     public static TikaInputStream get(File file) throws FileNotFoundException {
         return get(file, new Metadata());
     }
@@ -223,7 +255,11 @@ public class TikaInputStream extends TaggedInputStream {
      * @param metadata metadata instance
      * @return a TikaInputStream instance
      * @throws FileNotFoundException if the file does not exist
+     * or cannot be opened for reading
+     * @deprecated use {@link #get(Path, Metadata)}. In Tika 2.0,
+     * this will be removed or modified to throw an IOException.
      */
+    @Deprecated
     public static TikaInputStream get(File file, Metadata metadata)
             throws FileNotFoundException {
         metadata.set(Metadata.RESOURCE_NAME_KEY, file.getName());
@@ -320,9 +356,9 @@ public class TikaInputStream extends TaggedInputStream {
             throws IOException {
         // Special handling for file:// URIs
         if ("file".equalsIgnoreCase(uri.getScheme())) {
-            File file = new File(uri);
-            if (file.isFile()) {
-                return get(file, metadata);
+            Path path = Paths.get(uri);
+            if (Files.isRegularFile(path)) {
+                return get(path, metadata);
             }
         }
 
@@ -360,9 +396,9 @@ public class TikaInputStream extends TaggedInputStream {
         // Special handling for file:// URLs
         if ("file".equalsIgnoreCase(url.getProtocol())) {
             try {
-                File file = new File(url.toURI());
-                if (file.isFile()) {
-                    return get(file, metadata);
+                Path path = Paths.get(url.toURI());
+                if (Files.isRegularFile(path)) {
+                    return get(path, metadata);
                 }
             } catch (URISyntaxException e) {
                 // fall through
@@ -398,13 +434,13 @@ public class TikaInputStream extends TaggedInputStream {
     }
 
     /**
-     * The file that contains the contents of this stream. This is either
-     * the original file passed to the {@link #TikaInputStream(File)}
-     * constructor or a temporary file created by a call to the
-     * {@link #getFile()} method. If neither has been called, then
-     * the value is <code>null</code>.
+     * The path to the file that contains the contents of this stream.
+     * This is either the original file passed to the
+     * {@link #TikaInputStream(Path)} constructor or a temporary file created
+     * by a call to the {@link #getPath()} method. If neither has been called,
+     * then the value is <code>null</code>.
      */
-    private File file;
+    private Path path;
 
     /**
      * Tracker of temporary resources.
@@ -433,16 +469,32 @@ public class TikaInputStream extends TaggedInputStream {
      */
     private Object openContainer;
 
+    /**
+     * Creates a TikaInputStream instance. This private constructor is used
+     * by the static factory methods based on the available information.
+     *
+     * @param path the path to the file that contains the stream
+     * @throws IOException if an I/O error occurs
+     */
+    private TikaInputStream(Path path) throws IOException {
+        super(new BufferedInputStream(Files.newInputStream(path)));
+        this.path = path;
+        this.tmp = new TemporaryResources();
+        this.length = Files.size(path);
+    }
+
     /**
      * Creates a TikaInputStream instance. This private constructor is used
      * by the static factory methods based on the available information.
      *
      * @param file the file that contains the stream
      * @throws FileNotFoundException if the file does not exist
+     * @deprecated use {@link #TikaInputStream(Path)}
      */
+    @Deprecated
     private TikaInputStream(File file) throws FileNotFoundException {
         super(new BufferedInputStream(new FileInputStream(file)));
-        this.file = file;
+        this.path = file.toPath();
         this.tmp = new TemporaryResources();
         this.length = file.length();
     }
@@ -462,7 +514,7 @@ public class TikaInputStream extends TaggedInputStream {
     private TikaInputStream(
             InputStream stream, TemporaryResources tmp, long length) {
         super(stream);
-        this.file = null;
+        this.path = null;
         this.tmp = tmp;
         this.length = length;
     }
@@ -521,25 +573,20 @@ public class TikaInputStream extends TaggedInputStream {
     }
 
     public boolean hasFile() {
-        return file != null;
+        return path != null;
     }
 
-    public File getFile() throws IOException {
-        if (file == null) {
+    public Path getPath() throws IOException {
+        if (path == null) {
             if (position > 0) {
                 throw new IOException("Stream is already being read");
             } else {
                 // Spool the entire stream into a temporary file
-                file = tmp.createTemporaryFile();
-                OutputStream out = new FileOutputStream(file);
-                try {
-                    IOUtils.copy(in, out);
-                } finally {
-                    out.close();
-                }
+                path = tmp.createTempFile();
+                Files.copy(in, path, REPLACE_EXISTING);
 
                 // Create a new input stream and make sure it'll get closed
-                FileInputStream newStream = new FileInputStream(file);
+                InputStream newStream = Files.newInputStream(path);
                 tmp.addResource(newStream);
 
                 // Replace the spooled stream with the new stream in a way
@@ -554,16 +601,21 @@ public class TikaInputStream extends TaggedInputStream {
                     }
                 };
 
-                length = file.length();
+                length = Files.size(path);
             }
         }
-        return file;
+        return path;
+    }
+
+    /**
+     * @see #getPath()
+     */
+    public File getFile() throws IOException {
+        return getPath().toFile();
     }
 
     public FileChannel getFileChannel() throws IOException {
-        FileInputStream fis = new FileInputStream(getFile());
-        tmp.addResource(fis);
-        FileChannel channel = fis.getChannel();
+        FileChannel channel = FileChannel.open(getPath());
         tmp.addResource(channel);
         return channel;
     }
@@ -575,7 +627,7 @@ public class TikaInputStream extends TaggedInputStream {
     /**
      * Returns the length (in bytes) of this stream. Note that if the length
      * was not available when this stream was instantiated, then this method
-     * will use the {@link #getFile()} method to buffer the entire stream to
+     * will use the {@link #getPath()} method to buffer the entire stream to
      * a temporary file in order to calculate the stream length. This case
      * will only work if the stream has not yet been consumed.
      *
@@ -584,7 +636,7 @@ public class TikaInputStream extends TaggedInputStream {
      */
     public long getLength() throws IOException {
         if (length == -1) {
-            length = getFile().length();
+            getPath(); // updates length internally
         }
         return length;
     }
@@ -625,7 +677,7 @@ public class TikaInputStream extends TaggedInputStream {
 
     @Override
     public void close() throws IOException {
-        file = null;
+        path = null;
         mark = -1;
 
         // The close method was explicitly called, so we indeed
@@ -647,7 +699,7 @@ public class TikaInputStream extends TaggedInputStream {
     public String toString() {
         String str = "TikaInputStream of ";
         if (hasFile()) {
-            str += file.toString();
+            str += path.toString();
         } else {
             str += in.toString();
         }
